<?php

if (!defined('ABSPATH')) {
    exit;
}

class mbe_shipping_method extends WC_Shipping_Method {

    private $found_rates;

    public function __construct() {
        $this->id = WF_MBE_ID;
        $this->method_title = __('Mail Boxes', 'wf-shipping-mbe');
        $this->method_description = __('Obtains  real time shipping rates and Print shipping labels via MBE Shipping API.', 'wf-shipping-mbe');
        $this->helper = new Mbe_Shipping_Helper_Data();
        $this->logger = new Mbe_Shipping_Helper_Logger();
        $this->init();
    }

    // override to back compatibility with wooCommerce 2.0
    function is_available( $package ) {
        global $woocommerce;

        if ( isset( $woocommerce->cart->cart_contents_total ) && isset( $this->min_amount ) && $this->min_amount && $this->min_amount > $woocommerce->cart->cart_contents_total )
            return false;

        $ship_to_countries = '';

        if ( $this->availability == 'specific' ) :
            $ship_to_countries = $this->countries;
        else :
            if ( get_option( 'woocommerce_allowed_countries' ) == 'specific' ) :
                $ship_to_countries = get_option( 'woocommerce_specific_allowed_countries' );
            endif;
        endif;

        if ( is_array( $ship_to_countries ) ) :
            if ( ! in_array( $package['destination']['country'], $ship_to_countries ) ) return false;
        endif;

        return apply_filters( 'woocommerce_shipping_' . $this->id . '_is_available', true, $package );
    }

    private function init() {
        // Load the settings.
        $this->init_form_fields();
        $this->init_settings();

        add_action('woocommerce_update_options_shipping_' . $this->id, array($this, 'process_admin_options'));
    }

    public function init_form_fields() {
        $this->form_fields = include( 'data-mbe-settings.php' );
    }

    public function calculate_shipping($package=array()) {
        if (isset($package['destination']) && isset($package['destination']['country'])) {
            if ($this->helper->isEnabled() && $this->helper->enabledCountry($package['destination']['country'])) {
                $carrier = new Mbe_Shipping_Model_Carrier();
                $rates = $carrier->collectRates($package);

                foreach ($rates as $r) {
                    $rate = array(
                        'id' => $this->id . ':' . $r['method'],
                        'label' => $r['label'],
                        'cost' => $r['price'],
                        //                'calc_tax' => 'per_item'
                    );
                    // Register the rate
                    $this->add_rate($rate);
                }
            }
        }
    }

}
