<?php

include_once 'class-mbe-tracking-admin.php';

class mbe_tracking_factory
{

    const ITALIAN_URL = "https://www.mbe.it/it/tracking?c=";
    const SPAIN_URL = "https://www.mbe.es/es/tracking?c=";
    const GERMANY_URL = "http://www.mbe.de/sendung-verfolgen/?accio=track&param1=";
    const AUSTRIA_URL = "http://www.mbe.at/sendung-verfolgen/?accio=track&param1=";
    const FRANCE_URL = "http://www.mbefrance.fr/nc/utility-menu/suivi-de-votre-expedition/?accio=track&param1=";

    public static function create($order_id)
    {

        $logger = new Mbe_Shipping_Helper_Logger();
        $helper = new Mbe_Shipping_Helper_Data();
        $logger->log("CREATESHIPMENT");

        $order = new WC_Order($order_id);

        $shipmentConfigurationMode = $helper->getShipmentConfigurationMode();

        if ($shipmentConfigurationMode == Mbe_Shipping_Model_Carrier::SHIPMENT_CONFIGURATION_MODE_ONE_SHIPMENT_PER_ITEM) {

            foreach ($order->get_items() as $item) {
                $itemQty = $item['qty'];
                $id_product = $item['product_id'];

                $product = $helper->getProduct($id_product);

                $itemWeight = $product->weight;
                for ($i = 1; $i <= $itemQty; $i++) {
                    $qty = array();
                    $qty[$id_product] = 1;
                    mbe_tracking_factory::createSingleShipment($order, $itemWeight, 1, $qty, $product->price);
                }
            }
        }
        elseif ($shipmentConfigurationMode == Mbe_Shipping_Model_Carrier::SHIPMENT_CONFIGURATION_MODE_ONE_SHIPMENT_PER_SHOPPING_CART_SINGLE_PARCEL) {
            $maxPackageWeight = $helper->getMaxPackageWeight();
            $boxesWeights = array();
            $goodsValue = 0.0;

            foreach ($order->get_items() as $item) {
                $itemQty = $item['qty'];
                $id_product = $item['product_id'];
                $product = $helper->getProduct($id_product);
                $logger->logVar($product->price, "product price");

                for ($i = 1; $i <= $itemQty; $i++) {
                    $canAddToExistingBox = false;
                    for ($j = 0; $j <= count($boxesWeights); $j++) {
                        $val = isset($boxesWeights[$j]) ? $boxesWeights[$j] : 0;
                        $newWeight = $val + $product->weight;
                        if ($newWeight < $maxPackageWeight) {
                            $canAddToExistingBox = true;
                            $boxesWeights[$j] = $newWeight;
                            break;
                        }
                    }
                    if (!$canAddToExistingBox) {
                        $boxesWeights[] = $product->weight;;
                    }
                    $goodsValue = $goodsValue + $product->price;
                }
            }
            $numBoxes = count($boxesWeights);

            $logger->logVar($numBoxes, "boxes amount");
            $logger->logVar($boxesWeights, "boxes weights");
            $logger->logVar($goodsValue, "goods value");

            mbe_tracking_factory::createSingleShipment($order, $boxesWeights, $numBoxes, array(), $goodsValue);

        }
        elseif ($shipmentConfigurationMode == Mbe_Shipping_Model_Carrier::SHIPMENT_CONFIGURATION_MODE_ONE_SHIPMENT_PER_SHOPPING_CART_MULTI_PARCEL) {
            $boxesWeights = array();
            $numBoxes = 0;
            $goodsValue = 0.0;

            foreach ($order->get_items() as $item) {
                $numBoxes += $item['qty'];
                $id_product = $item['product_id'];
                $product = $helper->getProduct($id_product);
                $logger->logVar($product->price, "product price");
                for ($i = 1; $i <= $item['qty']; $i++) {
                    $boxesWeights[] = $product->weight;
                    $goodsValue = $goodsValue + $product->price;
                }
            }
            $logger->logVar($numBoxes, "boxes amount");
            $logger->logVar($boxesWeights, "boxes weights");
            $logger->logVar($goodsValue, "goods value");

            mbe_tracking_factory::createSingleShipment($order, $boxesWeights, $numBoxes, array(), $goodsValue);

        }

    }

    /**
     * @param $order WC_Order
     * @param $weight float
     * @param $boxes integer
     * @param array $qty
     */
    public static function createSingleShipment($order, $weight, $boxes, $qty, $goodsValue)
    {
        $logger = new Mbe_Shipping_Helper_Logger();
        $helper = new Mbe_Shipping_Helper_Data();
        $logger->log('CREATE SINGLE SHIPMENT');
        $logger->logVar(func_get_args(), 'CREATE SINGLE SHIPMENT ARGS');
        try {

            $shippingMethod = $helper->getShippingMethod($order);
            $serviceName = $helper->getServiceName($order);

            if ($shippingMethod) {
                $val = explode(':', $shippingMethod);
                $shippingMethod = $val[1];
                $shippingMethodArray = explode('_', $shippingMethod);

                $service = $shippingMethodArray[0];
                $subzone = $shippingMethodArray[1];

                if (version_compare(WC()->version, '3', '>=')) {
                    $firstName = $order->get_shipping_first_name();
                    $lastName = $order->get_shipping_last_name();
                    $address = trim($order->get_shipping_address_1() . ' ' . $order->get_shipping_address_2());
                    $phone = $order->get_billing_phone();
                    $city = $order->get_shipping_city();
                    $countryId = $order->get_shipping_country();
                    $region = $order->get_shipping_state();
                    $postCode = $order->get_shipping_postcode();
                    $email = $order->get_billing_email();
                }
                else {
                    $firstName = $order->shipping_first_name;
                    $lastName = $order->shipping_last_name;
                    $address = trim($order->shipping_address_1 . ' ' . $order->shipping_address_2);
                    $phone = $order->billing_phone;
                    $city = $order->shipping_city;
                    $countryId = $order->shipping_country;
                    $region = $order->shipping_state;
                    $postCode = $order->shipping_postcode;
                    $email = $order->billing_email;
                }

                $ws = new Mbe_Shipping_Model_Ws();
                $mbeShipment = $ws->createShipping($countryId, $region, $postCode, $weight, $boxes, $service, $subzone, $firstName, $lastName, $address, $phone, $city, $email, $goodsValue);

                $logger->logVar($mbeShipment, "MBE SHIPMENT");


                if ($mbeShipment) {
                    $trackingNumber = $mbeShipment->MasterTrackingMBE;
                    $label = $mbeShipment->Labels->Label;

                    if (version_compare(WC()->version, '3', '>=')) {
                        $orderId = $order->get_id();
                    }
                    else {
                        $orderId = $order->id;
                    }

                    $fileName = 'MBE_' . $orderId . '_' . $trackingNumber;
                    mbe_tracking_factory::saveShipmentDocument($label->Type, $label->Stream, $fileName);
                    mbe_tracking_factory::saveMultipleShipmentInfo($orderId, woocommerce_mbe_tracking_admin::SHIPMENT_SOURCE_TRACKING_NUMBER, $trackingNumber);
                    update_post_meta($orderId, woocommerce_mbe_tracking_admin::SHIPMENT_SOURCE_TRACKING_NUMBER, $trackingNumber, true);
                    update_post_meta($orderId, woocommerce_mbe_tracking_admin::SHIPMENT_SOURCE_TRACKING_NAME, $serviceName, true);
                    update_post_meta($orderId, woocommerce_mbe_tracking_admin::SHIPMENT_SOURCE_TRACKING_SERVICE, $service, true);
                    update_post_meta($orderId, woocommerce_mbe_tracking_admin::SHIPMENT_SOURCE_TRACKING_ZONE, $subzone, true);
                    mbe_tracking_factory::saveMultipleShipmentInfo($orderId, woocommerce_mbe_tracking_admin::SHIPMENT_SOURCE_TRACKING_FILENAME, $fileName . '.' . strtolower($label->Type), true);
                    update_post_meta($orderId, woocommerce_mbe_tracking_admin::SHIPMENT_SOURCE_TRACKING_URL, mbe_tracking_factory::getTrackingUrlBySystem());
                }
            }

        }
        catch (Exception $e) {
            $logger->log($e->getMessage());
        }
    }

    public static function saveMultipleShipmentInfo($post_id, $key, $value)
    {
        $old_meta = get_post_meta($post_id, $key, true);
        // Update post meta
        if (!empty($old_meta)) {
            $old_meta = $old_meta . '--' . $value;
            update_post_meta($post_id, $key, $old_meta);
        }
        else {
            add_post_meta($post_id, $key, $value, true);
        }
    }


    public static function saveShipmentDocument($type, $content, $filename)
    {
        $helper = new Mbe_Shipping_Helper_Data();
        $logger = new Mbe_Shipping_Helper_Logger();

        $ext = "txt";
        if ($type == "HTML") {
            $ext = "html";
        }
        elseif ($type == "PDF") {
            $ext = "pdf";
        }
        $filePath = $helper->getShipmentFilePath($filename, $ext);
        $saveResult = file_put_contents($filePath, $content);

        $message = "Saving shipping document :" . $filePath;

        if ($saveResult) {
            $message .= " OK";
        }
        else {
            $message .= " FAILURE";
        }

        $logger->log($message);
    }

    private static function getTrackingUrlBySystem()
    {
        $helper = new Mbe_Shipping_Helper_Data();
        $system = $helper->getCountry();

        $result = "";
        if ($system == "IT") {
            $result = self::ITALIAN_URL;
        }
        elseif ($system == "ES") {
            $result = self::SPAIN_URL;
        }
        elseif ($system == "DE") {
            $result = self::GERMANY_URL;
        }
        elseif ($system == "AT") {
            $result = self::AUSTRIA_URL;
        }
        elseif ($system == "FR") {
            $result = self::FRANCE_URL;
        }
        return $result;
    }

}

?>